class IntTree {
    
    private class Node {
        int elem;
        Node left = null;
        Node right = null;
        
        Node(int elem) {
            this.elem = elem;
        }
        
        void add(int elem) {
            if (elem < this.elem) {
                if (this.left == null) {
                    this.left = new Node(elem);
                } else {
                    this.left.add(elem);
                }
            } else {
                if (this.right == null) {
                    this.right = new Node(elem);
                } else {
                    this.right.add(elem);
                }
            }
        }
        
        int countNodes() {
            if (this.left == null && this.right == null) { return 1; }

            if (this.left == null) { return 1 + this.right.countNodes(); }
            if (this.right == null) { return 1 + this.left.countNodes(); }

            return 1 + this.left.countNodes() + this.right.countNodes();
        }
        
        int countLeaves() {
            if (this.left == null && this.right == null) { return 1; }

            return (this.left == null ? 0 : this.left.countLeaves()) + (this.right == null ? 0 : this.right.countLeaves());
        }
        
        int height() {
            if (this.left == null && this.right == null) { return 1; }

            if (this.left == null) { return 1 + this.right.height(); }
            if (this.right == null) { return 1 + this.left.height(); }

            return 1 + Math.max(this.left.height(), this.right.height());
        }
        
        void printLeaves() {
            if (this.left == null && this.right == null) {
                System.out.println(this.elem);
            }
            if (this.left != null) {
                this.left.printLeaves();
            }
            if (this.right != null) {
                this.right.printLeaves();
            }
        }
        
        void printInOrderUp() {
            if (this.left != null) {
                this.left.printInOrderUp();
            }
            System.out.println(this.elem);
            if (this.right != null) {
                this.right.printInOrderUp();
            }
        }
        
        void printInOrderUpSub(int elem) {
            if (elem == this.elem) {
                this.printInOrderUp();
            } else if (elem > this.elem) {
                this.right.printInOrderUpSub(elem);
            } else {
                this.left.printInOrderUpSub(elem);
            }

        }
    
        void printPostOrder() {
            if (this.left != null) {
                this.left.printPostOrder();
            }
            if (this.right != null) {
                this.right.printPostOrder();
            }
            System.out.println(this.elem);
        }
    
        void printPreOrder() {
            System.out.println(this.elem);
            if (this.left != null) {
                this.left.printPreOrder();
            }
            if (this.right != null) {
                this.right.printPreOrder();
            }
        }
    }
    
    private Node root = null;
    
    public void add(int elem) {
        if (empty()) {
            this.root = new Node(elem);
        } else {
            this.root.add(elem);
        }
    }
    
    public boolean empty() {
        return this.root == null;
    }
    
    public int countNodes() {
        return this.empty() ? 0 : this.root.countNodes();
    }
    
    public int countLeaves() {
        return this.root == null ? 0 : this.root.countLeaves();
    }
    
    public int height() {
        return this.root == null ? 0 : this.root.height();
    }
    
    public void printLeaves() {
        if (this.root != null) {
            this.root.printLeaves();
        }
    }
    
    public void printInOrderUp() {
        if (this.root != null) {
            this.root.printInOrderUp();
        }
    }
    
    public void printInOrderUpSub(int elem) {
        if (this.root != null) {
            this.root.printInOrderUpSub(elem);
        }
    }
    
    public void printPostOrder(){
        if (this.root != null) {
            this.root.printPostOrder();
        }
    }
    
    public void printPreOrder(){
        if (this.root != null) {
            this.root.printPreOrder();
        }
    }

    public static void main(String[] args) {
        // Aufgabe4.main(new String[0]);
    }
}
